﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Windows.Forms;
using System.Audio;
using BrawlLib.IO;
using BrawlLib.Wii.Audio;
using StandaloneBRSTMBuilder;

namespace StandaloneBrstmBuilder
{
    static class Program
    {
        public static string TITLE = "Standalone BRSTM Builder";
        public static string USAGE = "Usage: BrstmBuilder.exe input_file output_file [loop_start [loop_end]]\n" +
            "Loop start and end values are optional. By default, a song will loop from start to end." +
            "Use a loop start of -1 to create a non-looping song.";
        private static int usage()
        {
            MessageBox.Show(USAGE, TITLE);
            return 1;
        }

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static unsafe int Main(String[] args)
        {
            Form1 f1 = new Form1();
            f1.Show();
            FileMap audioData = null;
            string output = "out_failsafe.brstm"; // this value should be replaced
            if (args.Length == 0)
            {
                OpenFileDialog open = new OpenFileDialog();
                open.Filter = ".wav files|*.wav";
                if (open.ShowDialog() == DialogResult.OK)
                {
                    SaveFileDialog save = new SaveFileDialog();
                    save.Filter = ".brstm files|*.brstm";
                    if (save.ShowDialog() == DialogResult.OK)
                    {
                        output = save.FileName;
                        BrstmConverterDialog dlg = new BrstmConverterDialog();
                        dlg.AudioSource = open.FileName;
                        if (dlg.ShowDialog() == DialogResult.OK)
                        {
                            audioData = dlg.AudioData;
                        }
                    }
                }
            }
            else if (args.Length == 1)
            {
                if (args[0] == "--help" || args[0] == "/?")
                {
                    Console.WriteLine(USAGE);
                    return 0;
                }
                else
                {
                    return usage();
                }
            }
            else
            {
                // Process arguments
                string input;
                int loopStart, loopEnd;
                if (args.Length == 2)
                {
                    input = args[0];
                    output = args[1];
                    loopStart = 0;
                    loopEnd = -1; // loop whole song
                }
                else if (args.Length == 3)
                {
                    input = args[0];
                    output = args[1];
                    loopEnd = -1;
                    try
                    {
                        loopStart = Convert.ToInt32(args[2]);
                    }
                    catch (FormatException)
                    {
                        return usage();
                    }
                }
                else if (args.Length == 4)
                {
                    input = args[0];
                    output = args[1];
                    try
                    {
                        loopStart = Convert.ToInt32(args[2]);
                        loopEnd = Convert.ToInt32(args[3]);
                    }
                    catch (FormatException)
                    {
                        return usage();
                    }
                }
                else
                {
                    return usage();
                }
                System.Console.WriteLine("args.Length: " + args.Length);
                System.Console.WriteLine("loopStart: " + loopStart + "; loopEnd: " + loopEnd);
                // Get audio stream
                using (IAudioStream _sourceStream = WAV.FromFile(input))
                {
                    // It is not necessary to adjust for the block size (14336) here - BrawlLib does that in RSTMConverter.
                    if (loopEnd == -1)
                    {
                        loopEnd = _sourceStream.Samples;
                    }

                    if (loopStart == -1)
                    {
                        _sourceStream.IsLooping = false;
                    }
                    else
                    {
                        _sourceStream.IsLooping = true;
                        _sourceStream.LoopStartSample = loopStart;
                        _sourceStream.LoopEndSample = loopEnd;
                    }

                    using (ProgressWindow progress = new ProgressWindow(null, "Brstm Converter", "Encoding, please wait...", false))
                    {
                        audioData = RSTMConverter.Encode(_sourceStream, progress);
                    }
                }
            }

            if (audioData != null)
            {
                using (FileStream stream = new FileStream(output, FileMode.Create))
                {
                    using (BinaryWriter writer = new BinaryWriter(stream))
                    {
                        byte* b = (byte*)audioData.Address;
                        for (int i = 0; i < audioData.Length; i++)
                        {
                            writer.Write(b[i]);
                        }
                    }
                }
                return 0;
            }
            else
            {
                // nothing to do, but no error occurred
                return 0;
            }

        }
    }
}
